;  Demo of creating a scrollable child dialog within a user sizeable window

.686p
.model  flat, stdcall
option  casemap :none   ; case sensitive
.nolist
include windows.inc
inv equ Invoke

soff Macro QuotedText:Vararg    ; returns offset to a string
Local LocalText
.data
LocalText db QuotedText,0
.code
EXITM <offset LocalText>
Endm

uselib  MACRO   libname
    include     libname.inc
    includelib  libname.lib
ENDM

uselib user32
uselib kernel32
uselib comctl32
uselib gdi32

;uselib shell32
;uselib comdlg32
;uselib masm32
;uselib debug
.listall

.data?
hParent  dd ? ; dialog handle
hChild   dd ? ; dialog2 handle
bhead    dd ? ; bottom header handle
srect    RECT <>    ; scratch
crect    RECT <>    ; child position
brect    RECT <>    ; bottom header position
SizeChild dd ?  ; total vertical pixels needed from child
ChildDel  dd ?  ; delta to subtract from height of parent to get position of bottom of child
bheadDel  dd ?  ; delta to subtract from height of parent to get position of top of bottom headers (footers) 
LineSize  dd ?  ; size of one line step (one control or whatever).  Set in dialog proc of child.
nPosMax   dd ?  ; maximum pixel to scroll up to on child using page or line movement
.data
FirstTime dd -1 ; first time flag for setup
.code
Program:
    inv InitCommonControls
    inv GetModuleHandle, NULL
    mov edx,offset DlgProc
    inv DialogBoxParam, eax, 1001, 0, edx, 0
    inv ExitProcess, eax
    
DlgProc proc uses edx hWnd:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
local NewHeight:dword,NewWidth:dword

    .if uMsg == WM_COMMAND
    .elseif uMsg==WM_INITDIALOG
        push hWnd
        pop hParent
        inv GetModuleHandle, NULL
        mov edx,offset DlgProc2
        inv CreateDialogParam,eax, 1002, hParent, edx, 0
    .elseif uMsg == WM_SIZE
        .if hChild  ; if child dialog is defined
            mov eax,lParam
            mov NewWidth,eax
            and NewWidth,0ffffh ; the new width
            shr eax,16          ; the new height
            mov NewHeight,eax
            .if FirstTime   ; set up sizes and position rectangles
                mov FirstTime,0     ; no longer the first time              
                
                ; set position rectangle and delta for bottom header (footer)
                inv GetDlgItem,hParent,1011 ; bottom headers
                mov bhead,eax
                inv GetWindowRect,eax,addr brect    ; get position 
                inv MapWindowPoints, NULL, hParent, addr brect, 2   ; convert to something we can use
                mov eax,brect.top
                sub brect.bottom,eax    ; height of bottom header will not change
                mov eax,brect.left
                sub brect.right,eax     ; width of bottom header will not change
                mov eax,NewHeight       ; compute delta bottom headers to window height
                sub eax,brect.top       ; which will be applied to any NewHeight to get
                mov bheadDel,eax        ; top of bottom headers
                
                ; set up position and size of child dialog
                inv GetDlgItem,hParent,1008  ;upper header
                inv GetWindowRect,eax,addr srect
                inv MapWindowPoints, NULL, hParent, addr srect, 2
                mov eax,srect.bottom
                add eax,2               ; border, or any other amount desired
                mov crect.top,eax       ; start child window right below header
                mov crect.left,0        ; top and left of child will not change
                inv GetDlgItem,hParent,1011  ;lower header
                inv GetWindowRect,eax,addr srect
                inv MapWindowPoints, NULL, hParent, addr srect, 2
                mov eax,srect.top       ; top of lower headers
                sub eax,crect.top       ; minus top of child dialog gives clearance available for child
                sub eax,2               ; give a little extra clearance if desired
                mov edx,NewHeight
                sub edx,eax             ; subtract out amount needed for child
                mov ChildDel,edx        ; save delta to subtract from NewHeight to get bottom of child

                mov eax,NewWidth
          ;      sub eax,10              ; scrollbar, etc.
                mov crect.right,eax

            .endif
            
            ; move bottom headers (footers)
            mov eax,NewHeight
            sub eax,bheadDel
            mov brect.top,eax       ; new top of bottom headers
            inv MoveWindow,bhead,brect.left,brect.top,brect.right,brect.bottom,TRUE
            
            inv GetDlgItem,hParent,1012  ;other lower header, done the long way without saving rect
            push eax    ; save handle
            inv GetWindowRect,eax,addr srect
            inv MapWindowPoints, NULL, hParent, addr srect, 2
            mov eax,srect.top
            sub srect.bottom,eax    ; height
            mov eax,srect.left
            sub srect.right,eax     ; width             
            mov eax,brect.top       ; same top as other bottom header
            mov srect.top,eax       ; set new top
            pop eax     ; get back handle of 1012
            inv MoveWindow,eax,srect.left,srect.top,srect.right,srect.bottom,TRUE

            ; fit child into parent window
            mov eax,NewHeight
            sub eax,ChildDel        ; new height of child
            mov crect.bottom,eax
            inv MoveWindow,hChild,crect.left,crect.top,crect.right,crect.bottom,TRUE

        .endif  
    .elseif uMsg == WM_CLOSE
        inv EndDialog,hParent,0
    .endif
    xor eax,eax         
    ret
DlgProc EndP
.data?
vs        SCROLLINFO <>
;SCROLLINFO STRUCT
;  cbSize        DWORD sizeof scrollinfo
;  fMask         DWORD SIF_ALL for page, pos, and range
;  nMin          DWORD minimum scrolling postion
;  nMax          DWORD maximum scrolling postion
;  nPage         DWORD page size (sizes scroll box)
;  nPos          DWORD position of scroll box
;  nTrackPos     DWORD doesn't seem to work for anything
;SCROLLINFO ENDS

TopOfAll = 12       ; topmost pixel wanted on child. Also used to create test controls on child
                    ; this could be zero if everything on child wanted. Do not set negative.
LastControl dd ?    ; ID of bottom most control on child. Set in MakeControls.
                    ; Used for computing bottommost pixel on child to show.

.code
    
DlgProc2 proc uses edx hWnd:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
Local yAmount:DWORD
    .if uMsg == WM_COMMAND
        .if wParam > 2000 && wParam < 2999 ; one of the test buttons we created
            mov eax,wParam
            add eax,1000
            inv GetDlgItem,hChild,eax
            inv SendMessage,eax,WM_SETTEXT,0,soff("Tested")
        .endif
    .elseif uMsg==WM_SIZE
        .if FirstTime==0
            mov eax,SizeChild       ; compute scroll bar upper limit from total pixels needed
            sub eax,crect.bottom    ; Size of Child - visible height of child + 1
            add eax,1
            mov nPosMax,eax         ; max when moving by one line or page
            mov eax,crect.bottom
            mov vs.nPage,eax        ; size of one page
            inv SetScrollInfo,hChild,SB_VERT,addr vs,TRUE
            
            mov eax,vs.nPos         ; check to see if scroll needs adjustment
            .if eax > nPosMax       ; child is scrolled up too much
                mov edx,SizeChild
                sub edx,crect.bottom
                .if eax > vs.nMin   ; otherwise, already scrolled down as much as possible
                    sub eax,nPosMax
                    sub vs.nPos,eax
                    inv ScrollWindow,hChild,0,eax,0,0
                .endif
            .endif
            
        .endif
    .elseif uMsg==WM_VSCROLL
        
        mov edx,wParam
        and edx,0ffffh      ; save scroll bar movement code in edx
        mov eax,vs.nPage    ; assume it's a page movement
        .if edx==SB_LINEUP || edx==SB_LINEDOWN
            mov eax,LineSize    ; nope, its move one line
        .endif
        mov yAmount,eax         ; save movement amount for page or line
        .if edx==SB_LINEDOWN    || edx==SB_PAGEDOWN  ; actually need to scroll screen up
            mov eax,nPosMax ; is there enough room left to move amount wanted
            sub eax,vs.nPos
            .if eax < yAmount   
                mov yAmount,eax ; no, just use amount available
            .endif
            neg yAmount     ; scroll negative (up) to bring next line into view
        .elseif edx==SB_LINEUP || edx==SB_PAGEUP
            mov eax,vs.nPos
            sub eax,vs.nMin ; is there enough room left to move amount wanted
            .if eax < yAmount   
                mov yAmount,eax ; no, just use amount available
            .endif
        .elseif edx==SB_THUMBTRACK  ; || dx==SB_THUMBPOSITION
            mov eax,wParam
            shr eax,16      ; new position of scroll box
            neg eax
            add eax,vs.nPos ; where the scroll box was
            mov yAmount,eax ; amount of move
        .else
            jmp NoScroll    ; unknown or unwanted, e.g. SB_ENDSCROLL or SB_THUMBPOSITION, don't move
        .endif
        mov eax,yAmount ; amount of move
        sub vs.nPos,eax
        inv ScrollWindow,hChild,0,eax,NULL,NULL
        inv SetScrollInfo,hChild,SB_VERT,addr vs,TRUE
        inv UpdateWindow,hChild
        NoScroll:
    .elseif uMsg== WM_INITDIALOG    ; set up sizes based upon controls in child dialog
        push hWnd
        pop  hChild
        call MakeSomeControls       ; go make some controls for testing
        mov vs.cbSize,sizeof SCROLLINFO
        mov vs.fMask,SIF_ALL
        mov vs.nMin,TopOfAll ; or top of topmost control, or zero for top of child
        mov vs.nPos,TopOfAll ; start at the first pixel wanted on child
        mov vs.nPage,20      ; this will be recomputed properly later
        ; get bottom pixel of child to show.  Will use bottommost control here but
        ; could use height of child if child is well formatted.
        inv GetDlgItem,hChild,LastControl   ; last control on child
        inv GetWindowRect,eax,addr srect
        inv MapWindowPoints, NULL, hChild, addr srect, 2
        mov eax,srect.bottom    ; last pixel needed
        mov SizeChild,eax   ; max needed (last pixel used on child)
        mov vs.nMax,eax     ; will reset later after size of child is known
        
        ; now compute size of one line (one click on up or down arrow) (or just hardcode as desired)
        push srect.bottom   ; save bottom of control above last control
        mov eax,LastControl
        dec eax
        inv GetDlgItem,hChild,eax   ; get step of one control + spacing height
        inv GetWindowRect,eax,addr srect
        inv MapWindowPoints, NULL, hChild, addr srect, 2
        pop eax                 ; bottom of bottomost control
        sub eax,srect.bottom    ; bottom of next higher control
        mov LineSize,eax

        inv SetScrollInfo,hChild,SB_VERT,addr vs,TRUE
        inv ShowScrollBar,hChild,SB_VERT,TRUE
        mov eax,vs.nMin ; mov window to correct initial position
        neg eax
        inv ScrollWindow,hChild,0,eax,NULL,NULL ; syncronize window to scrollbar
    .endif

    xor eax,eax
    ret
DlgProc2    endp


;-----------------------------------------------------------------------------------------------
;------------------  testing code to add controls to the test child dialog ---------------------
.data?
buff db 10 dup (?)
temp1   dd ?
TestNum dd ?
hFnt    dd ?
.code

MaxTests    equ 50

MakeSomeControls:
    inv SendMessage,hParent,WM_GETFONT,0,0  ; get the font for the controls to use
    mov hFnt,eax
    mov TestNum,1   ; step through ids 1-MaxTests
LPP:
    call MakeControls
    inc TestNum
    mov eax,MaxTests
    cmp TestNum,eax
    jbe LPP
    ret

.data

Wide1 = 50      ; width of Test Button
Wide2 = 212     ; width of Description
Wide3 = 75      ; width of Clock Cycles
Wide4 = 75      ; width of Average Clocks
Wide5 = 75      ; width of scratch
Left1 = 11      ; Button   2001, 2002, .... , 2020   
Left2 = Left1+Wide1+4       ; Edit 1, Description, will compute location, 3001, 3002, ... , 3020
Left3 = Left2+Wide2+2       ; Edit 2, Clock Cycles, 4001, ...
Left4 = Left3+Wide3+2       ; Edit 3, Average Clocks 5001, ...
Left5 = Left4+Wide4+2       ; Edit 4, Test routine scratch for output value if desired, 6001, ...

Top  dd TopOfAll    ; starting top for controls, will increment by TDel
TDel =  32      ; Delta between rows
Ht   dd 32      ; height of rows
blank   db ' ',0
edt     db 'EDIT',0,0
CtlGp   dd  2000    ; base id

.code
MakeControls proc
    inv wsprintf,addr buff,soff("Test %i"),TestNum   ; get button caption
    mov eax,2000        ; each starts with id 2000+TestNum
    add eax,TestNum
    mov LastControl,eax ; save location of last control for dialog resizer code
    mov CtlGp,eax
    inv CreateWindowEx,0,soff("BUTTON"),addr buff,
            WS_CHILD or WS_VISIBLE or WS_TABSTOP,
            Left1,Top,Wide1,Ht,hChild,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,addr blank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_NOHIDESEL or ES_MULTILINE,
            Left2,Top,Wide2,Ht,hChild,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,addr blank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left3,Top,Wide3,Ht,hChild,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,addr blank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left4,Top,Wide4,Ht,hChild,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE
    add CtlGp,1000
    
    inv CreateWindowEx,WS_EX_CLIENTEDGE,addr edt,addr blank,
            WS_VISIBLE or WS_CHILDWINDOW or ES_AUTOHSCROLL or ES_NOHIDESEL,
            Left5,Top,Wide5,Ht,hChild,CtlGp,temp1,NULL
    inv SendMessage,eax,WM_SETFONT,hFnt,TRUE

    add Top,TDel    ; set up for next group

    ret
MakeControls EndP

End Program